/**
 * Copyright 2018 The AMP HTML Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS-IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import anyTest, { TestInterface } from 'ava';
import { Element } from '../../worker-thread/dom/Element';
import { createDocument, Document } from '../../worker-thread/dom/Document';

const test = anyTest as TestInterface<{
  document: Document;
  node: Element;
}>;

test.beforeEach(t => {
  const document = createDocument();

  t.context = {
    document,
    node: document.createElement('div'),
  };
});

test('detached node returns default value', async t => {
  const { node } = t.context;

  t.deepEqual(await node.getBoundingClientRectAsync(), {
    top: 0,
    right: 0,
    bottom: 0,
    left: 0,
    width: 0,
    height: 0,
    x: 0,
    y: 0,
  });
});

test('attached node returns default value in node environment', async t => {
  const { document, node } = t.context;

  document.body.appendChild(node);
  t.deepEqual(await node.getBoundingClientRectAsync(), {
    top: 0,
    right: 0,
    bottom: 0,
    left: 0,
    width: 0,
    height: 0,
    x: 0,
    y: 0,
  });
});
